package com.ehsure.honeywellscanner;

import android.app.Activity;
import android.util.Log;

import androidx.annotation.NonNull;

import com.honeywell.aidc.AidcManager;
import com.honeywell.aidc.BarcodeFailureEvent;
import com.honeywell.aidc.BarcodeReadEvent;
import com.honeywell.aidc.BarcodeReader;
import com.honeywell.aidc.ScannerNotClaimedException;
import com.honeywell.aidc.ScannerUnavailableException;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BasicMessageChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.StringCodec;

/**
 * HoneywellscannerPlugin
 */
public class HoneywellscannerPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private static BarcodeReader barcodeReader;
    private AidcManager manager;
    private BasicMessageChannel<String> mScannerChannel;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        mScannerChannel = new BasicMessageChannel<>(flutterPluginBinding.getBinaryMessenger(), "Honeywell_scanner", StringCodec.INSTANCE);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        Log.e("sky", call.method);
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        mScannerChannel.setMessageHandler(null);
    }

    @Override
    public void onAttachedToActivity(ActivityPluginBinding binding) {
        Activity activity = binding.getActivity();
        AidcManager.create(activity, aidcManager -> {
            manager = aidcManager;
            barcodeReader = manager.createBarcodeReader();
            try {
                barcodeReader.claim();
            } catch (ScannerUnavailableException e) {
                Log.e("sky", "Failed to claim scanner");
            }
            barcodeReader.addBarcodeListener(new BarcodeReader.BarcodeListener() {
                @Override
                public void onBarcodeEvent(BarcodeReadEvent barcodeReadEvent) {
                    activity.runOnUiThread(() -> {
                        String barcodeData = barcodeReadEvent.getBarcodeData();
                        mScannerChannel.send(barcodeData, reply -> Log.d("scanValue", reply));
                    });
                }

                @Override
                public void onFailureEvent(BarcodeFailureEvent barcodeFailureEvent) {
                    try {
                        barcodeReader.softwareTrigger(false);
                    } catch (ScannerNotClaimedException | ScannerUnavailableException e) {
                        e.printStackTrace();
                    }
                }
            });
        });
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {
        if (barcodeReader != null) {
            // close BarcodeReader to clean up resources.
            barcodeReader.close();
            barcodeReader = null;
        }
        if (manager != null) {
            // close AidcManager to disconnect from the scanner service.
            // once closed, the object can no longer be used.
            manager.close();
        }
    }
}
